#--------------------------------------------------------------------------------
# cbr-tb.tcl
# Author: Jeremy Ethridge.
# Dates: June 29-July 5, 1999.
# Notes: A DS-RED script that uses CBR traffic agents and the Token Bucket policer.
#
#        
#                   
#                    
#                     
#   -----              ----           ------
#   |src|--------------|e1|-----------|dest|
#   -----   10 Mb      ----   10 Mb   ------
#            5 ms              5 ms
#                    
#                       
#                   
#                
#
#--------------------------------------------------------------------------------


set ns [new Simulator]

set cir0   64000
set pir0  128000
set dir0  256000
set rate0 256000

set cir1   32000
set pir1  192000
set dir1  256000
set rate1 384000

set cirA  128000
set pirA  256000
set dirA  512000

set testTime 85.0
set packetSize 1000


# Set up the network topology shown at the top of this file:
set s1 [$ns node]
set e1 [$ns node]
set dest [$ns node]


$ns simplex-link $s1 $e1 10Mb 5ms dsRED/edge
$ns simplex-link $e1 $s1 10Mb 5ms dsRED/edge
$ns simplex-link $e1 $dest 10Mb 5ms dsRED/edge
$ns simplex-link $dest $e1 10Mb 5ms dsRED/edge


set qS1E1 [[$ns link $s1 $e1] queue]
set qE1D [[$ns link $e1 $dest] queue]

# Set DS RED parameters from Source1 to Edge1
$qS1E1 meanPktSize $packetSize
$qS1E1 set numQueues_ 1
$qS1E1 setNumPrec 3
$qS1E1 addPolicyEntry [$s1 id] [$dest id] flowid 1 TSW3CMD 10 $cir0 $pir0 $dir0
$qS1E1 addPolicyEntry [$s1 id] [$dest id] flowid 2 TSW3CMD 10 $cir1 $pir1 $dir1
$qS1E1 addPolicerEntry TSW3CMD 10 11 12
$qS1E1 addPHBEntry 10 0 0
$qS1E1 addPHBEntry 11 0 1
$qS1E1 addPHBEntry 12 0 2
$qS1E1 configQ 0 0 20 40 0.02
$qS1E1 configQ 0 1 15 25 0.05
$qS1E1 configQ 0 2 10 20 0.10

# Set DS RED parameters from Edge1 to Dest
$qE1D meanPktSize $packetSize
$qE1D set numQueues_ 1
$qE1D setNumPrec 3
$qE1D addPolicyEntry PreMk any 10 11 12 TSW3CMD 10 $cirA $pirA $dirA
$qE1D addPolicerEntry TSW3CMD 10 11 12
$qE1D addPHBEntry 10 0 0
$qE1D addPHBEntry 11 0 1
$qE1D addPHBEntry 12 0 2
$qE1D configQ 0 0 20 40 0.02
$qE1D configQ 0 1 15 25 0.05
$qE1D configQ 0 2 10 20 0.10


# Set up one CBR connection between each source and the destination:
set udp0 [new Agent/UDP]
$ns attach-agent $s1 $udp0
set cbr0 [new Application/Traffic/CBR]
$cbr0 attach-agent $udp0
$cbr0 set packet_size_ $packetSize
$udp0 set packetSize_ $packetSize
$cbr0 set rate_ $rate0
$udp0 set fid_ 1
set null0 [new Agent/Null]
$ns attach-agent $dest $null0
$ns connect $udp0 $null0

set udp1 [new Agent/UDP]
$ns attach-agent $s1 $udp1
set cbr1 [new Application/Traffic/CBR]
$cbr1 attach-agent $udp1
$cbr1 set packet_size_ $packetSize
$udp1 set packetSize_ $packetSize
$cbr1 set rate_ $rate1
$udp1 set fid_ 2
set null1 [new Agent/Null]
$ns attach-agent $dest $null1
$ns connect $udp1 $null1


proc finish {} {
    global ns 
    exit 0
}


$ns at 0.0 "$cbr0 start"
$ns at 0.0 "$cbr1 start"
$ns at 20.0 "puts \"At 20.0\""
$ns at 20.0 "$qS1E1 printStats"
$ns at 20.0 "$qE1D printStats"
$ns at 40.0 "puts \"At 40.0\""
$ns at 40.0 "$qS1E1 printStats"
$ns at 40.0 "$qE1D printStats"
$ns at 60.0 "puts \"At 60.0\""
$ns at 60.0 "$qS1E1 printStats"
$ns at 60.0 "$qE1D printStats"
$ns at 80.0 "puts \"At 80.0\""
$ns at 80.0 "$qS1E1 printStats"
$ns at 80.0 "$qE1D printStats"
$ns at $testTime "$cbr0 stop"
$ns at $testTime "$cbr1 stop"
$ns at [expr $testTime + 1.0] "finish"

$ns run
